#include <avr/io.h>
#include <string.h>
#include <avr/boot.h>
#include <avr/pgmspace.h>
#include <util/crc16.h>
#include "fat16.h"
#include <stdlib.h>

typedef struct
{
	unsigned long dev_id;
	unsigned short app_version;
	unsigned short crc;
} bootldrinfo_t;


uint16_t startcluster;
uint16_t updatecluster; //is set when update is available
uint32_t filesize;
bootldrinfo_t current_bootldrinfo;

void (*app_start)(void) = 0x0000;

void check_file(void)
{
	//Check filesize
	
	if (filesize != FLASHEND - BOOTLDRSIZE + 1)
		return;

	bootldrinfo_t *file_bootldrinfo;
	fat16_readfilesector(startcluster, (FLASHEND - BOOTLDRSIZE + 1) / 512 - 1);
	file_bootldrinfo =  (bootldrinfo_t*) (uint8_t*) (fat_buf + (FLASHEND - BOOTLDRSIZE - sizeof(bootldrinfo_t) + 1) % 512);
	
	//Check DEVID
	if (file_bootldrinfo->dev_id != DEVID)
		return;
		
	//Check application version
	if (file_bootldrinfo->app_version <= current_bootldrinfo.app_version)
		return;
		
	current_bootldrinfo.app_version = file_bootldrinfo->app_version;
	updatecluster = startcluster;
}

void flash_update(void)
{
	uint16_t filesector, j;
	uint8_t i;
	uint16_t *lpword;
	uint16_t adr;
	
	for (filesector = 0; filesector < (FLASHEND - BOOTLDRSIZE + 1) / 512; filesector++)
	{
		lpword = (uint16_t*) fat_buf;
		fat16_readfilesector(updatecluster, filesector);
	
		for (i=0; i<(512 / SPM_PAGESIZE); i++)
		{
			adr = (filesector * 512) + i * SPM_PAGESIZE;
			boot_page_erase(adr);
			while (boot_rww_busy())
				boot_rww_enable();
			
			for (j=0; j<SPM_PAGESIZE; j+=2)
				boot_page_fill(adr + j, *lpword++);

			boot_page_write(adr);
			while (boot_rww_busy())
				boot_rww_enable();
		}
	}
}

int main(void)
{
	uint16_t i;
	uint8_t attrib;
	
	memcpy_P(&current_bootldrinfo, (uint16_t*) FLASHEND - BOOTLDRSIZE - sizeof(bootldrinfo_t) + 1, sizeof(bootldrinfo_t));
	if (current_bootldrinfo.app_version == 0xFFFF)
		current_bootldrinfo.app_version = 0; //application not flashed yet
	
	if (fat16_init() == 0)
	{
		for (i=0; i<512; i++)
		{
			startcluster = fat16_readRootDirEntry(i, &filesize, &attrib);
			
			if (startcluster == 0xFFFF)
				continue;

			check_file();
		}
		
		if (updatecluster)
			flash_update();
	}
	
	unsigned short flash_crc = 0xFFFF;
	
	unsigned short adr;
	
	for (adr=0; adr<FLASHEND - BOOTLDRSIZE + 1; adr++)
		flash_crc = _crc_ccitt_update(flash_crc, pgm_read_byte(adr));
		
	if (flash_crc == 0)
		app_start();
	else
		while (1);
}
